/***********************************************************************

	File:	utils.c
	Rev:	a-1
	Date:	04/18/93

	Copyright (c) 1993, 1999 by David M. Warme

************************************************************************

	Various utility routines.

************************************************************************

	Modification Log:

	a-1:	04/18/93	warme
		: Created.  Collected these routines into this file
		:  from other places.

************************************************************************/

#include "steiner.h"


/*
 * Global Routines
 */

void			fatal (char *);
void			init_tables (void);
void *			new (size_t);
void			print_mask (char *, bitmap_t *, int);


/*
 * Global Variables
 */

int8u			nbits [256];


/*
 * External References
 */

	/* none */


/*
 * Local Routines
 */

static void		init_nbits (void);

/*
 * This routine displays a fatal message and then dies!
 */

	void
fatal (

char *		msg		/* IN - message to display. */
)
{
	(void) fprintf (stderr, "%s\n", msg);
	(void) fflush (stderr);
	abort ();
}

/*
 * This routine performs all dynamic memory allocation for the program.
 * We test for out of memory condition here.
 */

	void *
new (

size_t		size		/* IN - size of chunk in bytes. */
)
{
void *		p;

	if (size EQ 0) {
		/* Avoid implementation-defined bahavior of malloc! */
		size = 1;
	}

	p = malloc (size);
	if (p EQ NULL) {
		(void) fprintf (stderr, "Out of memory!\n");
		exit (1);
	}

	return (p);
}

/*
 * This routine prints out a given subset.
 */

	void
print_mask (

char *		msg,		/* IN - header message to display */
bitmap_t *	bp1,		/* IN - subset to display */
int		n		/* IN - number of bits in mask */
)
{
int		i;
int		count;

	(void) printf ("%s", msg);
	count = 0;
	for (i = 0; i < n; i++) {
		if (BITON (bp1, i)) {
			if (++count > 20) {
				printf ("\n%s	", msg);
				count = 0;
			}
			printf (" %lu", (int32u) i);
		}
	}
	printf ("\n");
}

/*
 * This routine initializes various global tables.
 */

	void
init_tables (void)

{
	init_nbits ();
}


/*
 * This routine initializes a translation table that converts an 8-bit
 * byte into the count of ONE bits in the byte.
 */

	static
	void
init_nbits (void)

{
int		i;
int32u		bits;
int		count;

	for (i = 0; i < 256; i++) {
		count = 0;
		for (bits = ((int32u) i); bits > 0; bits >>= 1) {
			if ((bits & 0x01) NE 0) {
				++count;
			}
		}
		nbits [i] = ((int8u) count);
	}
}
